# ----------------------------------------------------------------------
# This file was generated by /Users/humberto.a.sanchez.ii/PycharmProjects/PyUt/venv-pyenv-3.8.5-alternate/bin/img2py
#
from wx.lib.embeddedimage import PyEmbeddedImage

embeddedImage: PyEmbeddedImage = PyEmbeddedImage(
    b'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAKnRFWHRDcmVhdGlvbiBUaW1l'
    b'AE1vIDMgTXJ6IDIwMDMgMjI6NDI6MTAgKzAxMDD/JlV7AAAAB3RJTUUH1AcHCBwd4e5wDAAA'
    b'AAlwSFlzAAAK8AAACvABQqw0mAAAAARnQU1BAACxjwv8YQUAAAIdSURBVHjatZPbTxNBFId/'
    b'O9t2S7EINKDB2oRgxYhakkZb45P+B8YXX7UJr6Y8+MS/gRohaqImokaMVzCEy4PxhlUf1JQg'
    b'XaCRSy0tXbZlL916drOikkbxwTP5kpk58+1szswA/x6czcZga9ELP2dg0Fl2COoL/RwG8IZm'
    b'ZX5L8g3s5FVu9PzRrkNCnbwjJWeOVCbwkjIrjr/KN9GMFYx0R2Ptw+lBVBgHxpjPgBGh7Dwz'
    b'17Ql6kOBB9tOYhdqafjzr67Cx2XxrDsS6xj5+hA6qyCZzJf0K8YQZV0EY22P67rCxfDbTs/+'
    b'e0IPLtOk30peQwMKGI5HY6GxhUfQUMb0VGG9dEG9izlkaM07Is+3n259EvB6vVllHi6P6+By'
    b'sNiCJSRxALfi0TOHxxefQuM0pKYlpdhL8owljxIJYpVXTpSOcy4lCI5DDXPC4WUduVblVPzY'
    b'2b0TmSGo1MQZSZUvqnfwZUOeNAtIGHwxrT2f9a1GGhtrdtM34HQwhAP7aj/kJqFUNMymJE2+'
    b'pN7GlCWPEebxZYkyrIItYh0pJBaa10INPrcfVNZMMQedNzAnSrrcT/JnSzB3fk18+yHDrngZ'
    b'OWoi3i81yZ3bm9wtvMCQFgv6Wp86gI+WMF5Nxi9HVkaedhGRyAqlPfm84ild1++TnLXlV0Rm'
    b's7z5Kpt9NxEkzEtST3yyq71cTa72FsyxYMtOQrKpKv8pfntx/zW+A1iI1PZ4wtPwAAAAAElF'
    b'TkSuQmCC')

